/*
 * Copyright (C) 2007 Martin Willi
 * HSR Hochschule fuer Technik Rapperswil
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.  See <http://www.fsf.org/copyleft/gpl.txt>.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 */

#include <stdlib.h>

#include "roam_job.h"

#include <sa/ike_sa.h>
#include <daemon.h>


typedef struct private_roam_job_t private_roam_job_t;

/**
 * Private data of an roam_job_t Object
 */
struct private_roam_job_t {
	/**
	 * public roam_job_t interface
	 */
	roam_job_t public;

#ifdef NETAPP
	host_t *host_addr;
	uint32_t vserverid;
	int roam_type;
#else /* !NETAPP */
	/**
	 * has the address list changed, or the routing only?
	 */
	bool address;
#endif /* NETAPP */
};

METHOD(job_t, destroy, void,
	private_roam_job_t *this)
{
#ifdef NETAPP
	if (this->host_addr)
	{
		this->host_addr->destroy(this->host_addr);
	}
#endif /* NETAPP */
	free(this);
}

METHOD(job_t, execute, job_requeue_t,
	private_roam_job_t *this)
{
	ike_sa_t *ike_sa;
	linked_list_t *list;
	ike_sa_id_t *id;
	enumerator_t *enumerator;
#ifdef NETAPP
	host_t *test_ike_host;
	uint32_t test_vserverid;
	bool need_destroy = false;
#endif /* NETAPP */

	/* enumerator over all IKE_SAs gives us no way to checkin_and_destroy
	 * after a DESTROY_ME, so we check out each available IKE_SA by hand. */
	list = linked_list_create();
	enumerator = charon->ike_sa_manager->create_enumerator(
												charon->ike_sa_manager, TRUE);
	while (enumerator->enumerate(enumerator, &ike_sa))
	{
		id = ike_sa->get_id(ike_sa);
		list->insert_last(list, id->clone(id));
	}
	enumerator->destroy(enumerator);

	while (list->remove_last(list, (void**)&id) == SUCCESS)
	{
		ike_sa = charon->ike_sa_manager->checkout(charon->ike_sa_manager, id);
		if (ike_sa)
		{
#ifdef NETAPP
			/* roam_type 0, 1 (true|false) go to actual roam call, fo
			 * roam_type 2 goes to address check and flushing
			 */
			need_destroy = false;
			if (this->roam_type == ROAM_DELADDR)
			{
				test_ike_host = ike_sa->get_my_host(ike_sa);
				test_vserverid = ike_sa->get_vserverid(ike_sa);
				if (test_ike_host->ip_equals(test_ike_host, this->host_addr) &&
					test_vserverid == this->vserverid)
				{
					DBG2(DBG_JOB, "roam_job.c Delete IKE SA of %H...%H of vserver %d", test_ike_host, ike_sa->get_other_host(ike_sa), test_vserverid);
					(void)ike_sa->delete(ike_sa, false);
					/* we always destroy this IKE SA after a graceful delete attempt */
					need_destroy = true;
				}
			} else {
				if (ike_sa->roam(ike_sa, this->roam_type) == DESTROY_ME)
				{
					need_destroy = true;
				}
			}

			if (need_destroy)
#else /* !NETAPP */
			if (ike_sa->roam(ike_sa, this->address) == DESTROY_ME)
#endif /* NETAPP */
			{
				charon->ike_sa_manager->checkin_and_destroy(
											charon->ike_sa_manager, ike_sa);
			}
			else
			{
				charon->ike_sa_manager->checkin(charon->ike_sa_manager, ike_sa);
			}
		}
		id->destroy(id);
	}
	list->destroy(list);
	return JOB_REQUEUE_NONE;
}

METHOD(job_t, get_priority, job_priority_t,
	private_roam_job_t *this)
{
	return JOB_PRIO_MEDIUM;
}

#ifdef NETAPP
/**
 * Creates a job to inform IKE SA Manager regarding address change
 *
 * @param host_addr		host address of IKE end point in this job
 * @param vserverid		vserver ID
 * @param roam_type		0 Routing_change
 *				1 address list changed
 *				2 an IKE address deleted
 * @return			initiate_ike_sa_job_t object
 */
roam_job_t *roam_job_create(host_t *host_addr, uint32_t vserverid, int roam_type)
{
	private_roam_job_t *this;

	INIT(this,
		.public = {
			.job_interface = {
				.execute = _execute,
				.get_priority = _get_priority,
				.destroy = _destroy,
			},
		},
		.host_addr = ((host_addr) ? host_addr->clone(host_addr) : NULL),
		.vserverid = vserverid,
		.roam_type = roam_type,
	);

	return &this->public;
}
#else /* !NETAPP */
/*
 * Described in header
 */
roam_job_t *roam_job_create(bool address)
{
	private_roam_job_t *this;

	INIT(this,
		.public = {
			.job_interface = {
				.execute = _execute,
				.get_priority = _get_priority,
				.destroy = _destroy,
			},
		},
		.address = address,
	);

	return &this->public;
}
#endif /* NETAPP */

